#File Name: game_engine.py
#Creator: Caitlyn Clabaugh
#Description: Will read text file formated correctly and create text adventure.

class State:
    def __init__(self, name, description):
        self.name = name
        self.description = description
        self.edges = []

class Edge:
    def __init__(self, from_node, commands, to_node, constraints=None):
        to_nodeList = to_node.split(",")
        self.from_node = from_node 
        self.commands = commands
        self.to_node = to_nodeList
        self.constraints = constraints
        self.constraintObjects = []

class Object:
    def __init__(self, location_node, name, desc, description, weight):
        self.location_node = location_node
        self.name = name
        self.desc = desc
        self.description = description
        self.weight = weight

class Constraint:
    def __init__(self, name, object_name, state_name):
        self.name = name
        self.object_name = object_name
        self.state_name = state_name

class Backpack:
    def __init__(self, max_weight, current_weight):
        self.max_weight = int(max_weight)
        self.current_weight = int(current_weight)
        self.items = {}

class Engine:
    def __init__(self, filename):
        self.counter = 0
        self.timer = None
        self.timerEdge= None
        self.health = 1
        self.filename = filename
        self.initial_state = None
        self.node_objects = {}
        self.backpack = None

    def load(self):
        self.states = {}
        edges = []
        cons = []
        
        #READ FILE
        fp = open(self.filename)
        for line in fp:
            
            #ELIMINATE COMMENTS & COLONS
            if len(line) == 0 or line[0] == '#':
                continue
            items = line.strip().split("::")

            #ACCORDING TO GAME TEXT FILE...
            
            #NODES/STATES --> (SELF.STATES/SELF.NODE_OBJECTS) DICTIONARY
            if items[0] == 'node':
                self.node_objects[items[1]] = []
                s = State(*items[1:])
                if self.initial_state is None:
                    self.initial_state = s
                self.states[items[1]] = s
                
            #EDGES --> (EDGES) LIST
            elif items[0] == 'edge':
                edges.append(Edge(*items[1:]))
                
            #CONSTRAINTS --> (CONS) LIST
            elif items[0] == 'constraint':
                cons.append(Constraint(*items[1:]))
                
            #OBJECTS --> (SELF.NODE_OBJECTS) DICTIONARY
            elif items[0] == 'object':
                o = Object(*items[1:])
                self.node_objects[items[1]].append(o)
                
            #BACKPACK --> SELF.BACKPACK
            elif items[0] == 'backpack':
                b = Backpack(*items[1:])
                if self.backpack is None:
                    self.backpack = b
                    
        #LINK STATES TO EDGES AND EDGES TO CONSTRAINTS
        for edge in edges:
            self.states[edge.from_node].edges.append(edge)
            if edge.constraints is not None:
                #multiple constraints possible
                for constraint in edge.constraints.split(","):
                    for newConstraint in cons:
                        if newConstraint.name == constraint:
                            edge.constraintObjects.append(newConstraint)

        #START ALL OBJECTS (KEYS) IN BACKPACK DICTIONARY AS FALSE (VALUE)
        for key, value in self.node_objects.items():
            for o in value:
                if self.backpack is not None:
                    self.backpack.items[o.name] = [False, o]

        
    def play(self):
        current_state = self.initial_state
        backpack = self.backpack
        objects = self.node_objects
        gameOver = False
        
        while True:
            #SET HEALTH SPECIFICS IN TXT FILE
            if self.health == 0:
                gameOver = True

            #DESCRITION OF NODE
            print(current_state.description)

            #USER COMMAND
            command = raw_input("Command> ")

            for edge in current_state.edges:
                commandFound = False
                
                #EDGE COMMAND
                if command in edge.commands:
                    canPass = True
                
                    #GO TO NODE
                    if edge.constraintObjects == []:
                        current_state = self.states[edge.to_node[0]]

                    else:
                        for constraint in edge.constraintObjects:
                            #MISSING CONSTRAINT
                            if constraint.object_name != "eval" and constraint.object_name != "exec":
                                if backpack.items[constraint.object_name][0] == False:
                                    print "You can't ", command ,", you need to do something else first..."
                                    canPass = False
                                    break
                            #EVAL
                            elif constraint.object_name == "eval":
                                if eval(constraint.state_name) != True:
                                    canPass = False      
                            #EXEC
                            else:
                                exec(constraint.state_name)
                        if len(edge.to_node)== 1 and canPass:  
                            current_state = self.states[edge.to_node[0]]
                            
                        #HIERARCHY
                        elif canPass and len(edge.to_node) > 1:
                            current_state = self.states[edge.to_node[1]]
                        elif len(edge.to_node) > 1:
                            current_state = self.states[edge.to_node[0]]
                        canPass = True 
                    commandFound = True

                #OBJECT COMMANDS
                elif commandFound is False:
                    for constraint in edge.constraintObjects:
                        if command == constraint.object_name:
                            for o in objects[edge.from_node]:
                                if o.name == command:
                                    if backpack.current_weight < backpack.max_weight:
                                        #MAKE OBJECT TRUE IN BACKPACK
                                        if backpack.items[constraint.object_name][0] == False:
                                            print "You've got ", o.description
                                            backpack.items[constraint.object_name][0] = True
                                            backpack.current_weight += 1
                                    else:
                                        print "Your backpack is full."
                                        
                    commandFound = True
                    
        #WHENEVER GAME IS OVER/THROUGH
        if gameOver:
            print "Game Over!"

#START PLAYING!
game = Engine("top_chef_trapped_in_atlantis.txt")
game.load()
game.play()
